<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use App\Models\CargoModel;
use App\Models\DetallePermisosModel;
use App\Models\PermisosModel;

class Cargo extends BaseController
{
    protected $cargo, $session, $roles, $reglas, $reglasCambia, $detalle_rol, $permisos, $permisosAsignados;

    public function __construct()
    {
        $this->cargo = new CargoModel();
        $this->detalle_rol = new DetallePermisosModel();
        $this->permisos = new PermisosModel();

        $this->session = session();

        helper(['form']);


        $this->reglas =
            [
                'nombre_cargo' =>
                [
                    'rules' => 'required|is_unique[cargo.nombre_cargo]',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ]
            ];

        $this->reglasCambia =
            [
                'nombre_cargo' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];
    }

    /* INICIO */
    public function index()
    {
        $cargo = $this->cargo->findAll();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


        $data = ['datos' => $cargo,'validacion_sw' => 0, 'permisos' => $permisos];
        $view =
            view('header', $data) .
            view('cargo/cargo', $data) .
            view('footer');
        return $view;
    }
    public function insertar()
    {
        if ($this->request->getMethod() == "post" && $this->validate($this->reglas)) {

            $this->cargo->save([
                'nombre_cargo' => strtoupper($this->request->getPost('nombre_cargo')),
                'estado_cargo' => 1
            ]);

            return redirect()->to(base_url() . 'cargo');
        } else {
            $cargo = $this->cargo->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


            $data = ['datos' => $cargo, 'validation' => $this->validator, 'validacion_sw' => 0, 'permisos' => $permisos];
            $view =
                view('header', $data) .
                view('cargo/cargo', $data) .
                view('footer');
            return $view;
        }
    }

    public function editar($id, $valid = null)
    {
        $cargo = $this->cargo->where('id_cargo', $id)->first();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


        if ($valid != null) {
            $data = ['datos' => $cargo, 'validation' => $valid, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('cargo/editar', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $cargo, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('cargo/editar', $data);
            echo view('footer');
        }
    }

    public function actualizar()
    {
        $id = $this->request->getPost('id_cargo');
        $nombre_cargo = $this->request->getPost('nombre_cargo');
        $datos = $this->cargo->where('id_cargo !=', $id)->where('nombre_cargo', $nombre_cargo)->first();
        if (!$datos) {

            if ($this->request->getMethod() == "post" && $this->validate($this->reglasCambia)) {

                $this->cargo->update($this->request->getPost('id_cargo'), [
                    'nombre_cargo' => strtoupper($this->request->getPost('nombre_cargo')),
                    'estado_cargo' => $this->request->getPost('estado_cargo')

                ]);

                return redirect()->to(base_url() . 'cargo');
            } else {

                return $this->editar($this->request->getPost('id_cargo'), $this->validator);
            }
        } else {
  
            $cargo = $this->cargo->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $cargo, 'validacion_sw' => 1, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('cargo/cargo', $data);
            echo view('footer');
        }
    }

    public function permisos($id_cargo)
    {
        $permisos_rol = $this->cargo->detalle_rol($id_cargo);

        $permisos_general= $this->permisos->findAll();
        $permisosAsignados = $this->detalle_rol->where('id_cargo', $id_cargo)->findAll();

        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


        $datos=array();
        foreach($permisosAsignados as $permisoAsignado){
            $datos[$permisoAsignado['id_permiso']]=true;
        }

        $data = ['datos' => $permisos_rol,'asignado' => $datos,'permisos_general' => $permisos_general, 'id_cargo'=> $id_cargo, 'permisos' => $permisos];
        $view =
            view('header', $data) .
            view('cargo/permisos', $data) .
            view('footer');
        return $view;
    }

    public function guardarpermisos()
    {

        if ($this->request->getMethod() == 'post') {

            $idRol = $this->request->getPost('id_cargo');
            $permisos = $this->request->getPost('permisos');

            $this->detalle_rol->where('id_cargo', $idRol)->delete();

            foreach ($permisos as $permiso) {
                $this->detalle_rol->save(['id_cargo' => $idRol, 'id_permiso' => $permiso]);
            }
            return redirect()->to(base_url() . "/cargo");
        }
    }

}
