<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use App\Models\CitaModel;
use App\Models\PacienteModel;
use App\Models\PersonalModel;
use App\Models\ServicioModel;
use App\Models\PrescripcionModel;
use App\Models\DetallePermisosModel;
use App\Models\PermisosModel;
use App\Models\MedicamentoModel;


use FPDF;
use Picqer\Barcode\BarcodeGeneratorPNG;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\Writer\PngWriter;


class cita extends BaseController
{
    protected $cita, $session, $roles, $reglas, $reglasCambia, $servicio, $paciente, $personal, $prescripcion, $reglas_prescripcion, $permisos, $detalle_rol, $medicamento;

    public function __construct()
    {
        $this->cita = new CitaModel();
        $this->servicio = new ServicioModel();
        $this->paciente = new PacienteModel();
        $this->personal = new PersonalModel();
        $this->prescripcion = new PrescripcionModel();
        $this->detalle_rol = new DetallePermisosModel();
        $this->permisos = new PermisosModel();
        $this->medicamento = new MedicamentoModel();
        $this->session = session();

        helper(['form']);


        $this->reglas =
            [
                'id_paciente' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'id_personal' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'id_servicio' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'motivo' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'fecha' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'hora' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];

        $this->reglasCambia =
            [
                'id_paciente' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'id_personal' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'id_servicio' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'motivo' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'fecha' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'hora' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ]
            ];


        $this->reglas_prescripcion = [
            'id_medicamento' =>
            [
                'rules' => 'required',
                'errors' => [
                    'required' => 'El campo {field} es obligatorio.',
                ]
            ],
            'dosis' =>
            [
                'rules' => 'required',
                'errors' => [
                    'required' => 'El campo {field} es obligatorio.',
                ]
            ],
            'indicaciones' =>
            [
                'rules' => 'required',
                'errors' => [
                    'required' => 'El campo {field} es obligatorio.',
                ]
            ]
        ];
    }

    /* INICIO */
    public function index()
    {
        // Get the personal ID from the session
        $idPersonal = $this->session->id_personal;

        // Fetch citas filtered by the personal ID
        $cita = $this->cita->datos($idPersonal);

        // Fetch other necessary data
        $servicio = $this->servicio->findAll();
        $paciente = $this->paciente->findAll();
        $personal = $this->personal->medicos();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        // Prepare the data for the view
        $data = [
            'datos' => $cita,
            'validacion_sw' => 0,
            'servicios' => $servicio,
            'pacientes' => $paciente,
            'medicos' => $personal,
            'permisos' => $permisos
        ];

        // Return the combined view
        return view('header', $data) . view('cita/cita', $data) . view('footer');
    }



    public function lista_consultas()
    {
        // Get the personal ID from the session
        $idPersonal = $this->session->id_personal;

        // Fetch citas filtered by the personal ID
        $cita = $this->cita->lista_general_citas();

        // Fetch other necessary data
        $servicio = $this->servicio->findAll();
        $paciente = $this->paciente->findAll();
        $personal = $this->personal->medicos();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        // Prepare the data for the view
        $data = [
            'datos' => $cita,
            'validacion_sw' => 0,
            'servicios' => $servicio,
            'pacientes' => $paciente,
            'medicos' => $personal,
            'permisos' => $permisos
        ];

        // Return the combined view
        return view('header', $data) . view('cita/lista_consultas', $data) . view('footer');
    }

    



    public function insertar()
    {
        $folio = uniqid();
        if ($this->request->getMethod() == "post" && $this->validate($this->reglas)) {

            $this->cita->save([
                'folio' => $folio,
                'fecha' => $this->request->getPost('fecha'),
                'hora' => $this->request->getPost('hora'),
                'id_paciente' => $this->request->getPost('id_paciente'),
                'id_personal' => $this->request->getPost('id_personal'),
                'id_servicio' => $this->request->getPost('id_servicio'),
                'motivo' => $this->request->getPost('motivo'),
                'estado_cita' => 1
            ]);

            return redirect()->to(base_url() . 'cita');
        } else {
            $cita = $this->cita->datos();
            $servicio = $this->servicio->findAll();
            $paciente = $this->paciente->findAll();
            $personal = $this->personal->medicos();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


            $data = ['datos' => $cita, 'validacion_sw' => 0, 'servicios' => $servicio, 'pacientes' => $paciente, 'medicos' => $personal, 'validation' => $this->validator];
            $view =
                view('header', $data) .
                view('cita/cita', $data) .
                view('footer');
            return $view;
        }
    }

    public function editar($id, $valid = null)
    {
        $cita = $this->cita->where('id_cita', $id)->first();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);
        $servicio = $this->servicio->findAll();
        $paciente = $this->paciente->findAll();
        $personal = $this->personal->medicos();

        if ($valid != null) {
            $data = ['datos' => $cita, 'validation' => $valid, 'permisos' => $permisos, 'servicios' => $servicio, 'pacientes' => $paciente, 'medicos' => $personal];
            echo view('header', $data);
            echo view('cita/editar', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $cita, 'permisos' => $permisos, 'servicios' => $servicio, 'pacientes' => $paciente, 'medicos' => $personal];
            echo view('header', $data);
            echo view('cita/editar', $data);
            echo view('footer');
        }
    }

    public function actualizar()
    {


        if ($this->request->getMethod() == "post" && $this->validate($this->reglasCambia)) {

            $this->cita->update($this->request->getPost('id_cita'), [
                'fecha' => $this->request->getPost('fecha'),
                'hora' => $this->request->getPost('hora'),
                'id_paciente' => $this->request->getPost('id_paciente'),
                'id_personal' => $this->request->getPost('id_personal'),
                'id_servicio' => $this->request->getPost('id_servicio'),
                'motivo' => $this->request->getPost('motivo'),

            ]);

            return redirect()->to(base_url() . 'cita');
        } else {

            return $this->editar($this->request->getPost('id_cita'), $this->validator);
        }
    }

    public function atencion_cita($id, $valid = null)
    {
        $cita = $this->cita->datos_ID($id);
        $prescripcion = $this->prescripcion->datos($id);
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);
        $medicamento = $this->medicamento->findAll();

        if ($valid != null) {
            $data = ['datos' => $cita, 'datosprescripcion' => $prescripcion, 'validation' => $valid, 'permisos' => $permisos, 'medicamentos' => $medicamento];

            echo view('header', $data);
            echo view('cita/atencion_cita', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $cita, 'datosprescripcion' => $prescripcion, 'permisos' => $permisos, 'medicamentos' => $medicamento];

            echo view('header', $data);
            echo view('cita/atencion_cita', $data);
            echo view('footer');
        }
    }




    public function ticket_receta($id_cita)
    {
        // Crear un nuevo documento PDF en formato A4
        $pdf = new FPDF('P', 'mm', 'A4');
        $pdf->AddPage();

        // Establecer márgenes
        $pdf->SetMargins(0, 0, 0); // Sin márgenes (ajusta según sea necesario)

        // Establecer el título
        $pdf->SetTitle("Comprobante");

        $db = \Config\Database::connect();
        $query = $db->query("SELECT * FROM prescripcion pre 
                                INNER JOIN cita ci on ci.id_cita = pre.id_cita
                                INNER JOIN paciente pa on pa.id_paciente = pre.id_paciente
                                INNER JOIN personal pe on pe.id_personal = pre.id_personal
                                INNER  JOIN medicamento me ON me.id_medicamento = pre.id_medicamento 
                                WHERE pre.id_cita = $id_cita");

        $resultados = $query->getResultArray();

        // Obtén los datos del paciente
        foreach ($resultados as $dato) {

            $medico =       $dato['nombre_personal'];
            $nombre_paciente =       $dato['nombre'];
            $fecha =       $dato['fecha'];
        }

        // Generar el PDF
        $pdf->SetFont('Arial', 'B', 16);
        $pdf->Cell(0, 10, '       RECETA', 0, 1);
        $pdf->Ln(5);

        $pdf->SetFont('Arial', '', 12);
        $pdf->Cell(0, 10, 'PACIENTE : ' . $nombre_paciente, 0, 1);
        $pdf->Cell(0, 10, 'FECHA : ' . $fecha, 0, 1);
        $pdf->Cell(0, 10, 'MEDICO : ' . $medico, 0, 1);
        $pdf->Ln(5);

        // Generar el código de barras
        $generator = new BarcodeGeneratorPNG();
        $barcodeImage = WRITEPATH . 'cache/codigo_barras.png';
        file_put_contents($barcodeImage, $generator->getBarcode($id_cita, $generator::TYPE_CODE_128));

        // Insertar el código de barras en el PDF
        $pdf->Image($barcodeImage, 5, $pdf->GetY(), 60, 20); // Ajusta la posición y tamaño
        $pdf->Ln(20);
        // Mostrar el folio debajo del código de barras
        $pdf->SetFont('Arial', '', 12);
        $pdf->Cell(0, 5, '                  codigo: ' . $id_cita, 0, 1, 1); // Texto centrado debajo del código de barras
        $pdf->Ln(5);


        foreach ($resultados as $dato) {


            // Diagnóstico
            $pdf->SetFont('Arial', 'B', 12);
            $pdf->Cell(0, 10, 'MEDICAMENTO:', 0, 1);
            $pdf->SetFont('Arial', '', 12);
            $pdf->MultiCell(0, 10, $dato['nombre_medicamento']);

            $pdf->SetFont('Arial', 'B', 12);
            $pdf->Cell(0, 10, 'DOSIS:', 0, 1);
            $pdf->SetFont('Arial', '', 12);
            $pdf->MultiCell(0, 10, $dato['dosis']);

            $pdf->SetFont('Arial', 'B', 12);
            $pdf->Cell(0, 10, 'INDICACIONES:', 0, 1);
            $pdf->SetFont('Arial', '', 12);
            $pdf->MultiCell(0, 10, $dato['indicaciones']);


            $pdf->Ln(5);
        }
        // Limpiar las imágenes generadas
        if (file_exists($barcodeImage)) {
            unlink($barcodeImage);
        }
        /*   if (file_exists($qrImagePath)) {
            unlink($qrImagePath);
        } */

        $this->response->setHeader('Content-Type', 'application/pdf');
        $pdf->Output("ticket.pdf", "I");
    }



    public function ticket($id_cita)
    {
        // Crear un nuevo documento PDF en formato A4
        $pdf = new FPDF('P', 'mm', 'A4');
        $pdf->AddPage();

        // Establecer márgenes
        $pdf->SetMargins(0, 0, 0); // Sin márgenes (ajusta según sea necesario)

        // Establecer el título
        $pdf->SetTitle("Comprobante");

        $db = \Config\Database::connect();
        $query = $db->query("SELECT * FROM paciente pa inner join cita c on pa.id_paciente = c.id_paciente
                                    inner join personal pe on pe.id_personal = c.id_personal
                                    inner join servicios serv on serv.id_servicio = c.id_servicio
                                    where c.id_cita = '$id_cita'");
        $resultados = $query->getResultArray();

        // Obtén los datos del paciente
        foreach ($resultados as $dato) {
            $nombre =  $dato['nombre'];
            $fecha = $dato['fecha'] . ' - ' . $dato['hora'];
            $motivo = $dato['motivo'];
            $medico = $dato['nombre_personal'];
            $hora = $dato['hora'];
            $idcita = $dato['id_cita'];
            $folio = $dato['folio'];
        }

        // Generar el PDF
        $pdf->SetFont('Arial', 'B', 16);
        $pdf->Cell(0, 10, '       TICKET', 0, 1);
        $pdf->Ln(5);

        $pdf->SetFont('Arial', '', 12);
        $pdf->Cell(0, 10, 'PACIENTE : ' . $nombre, 0, 1);
        $pdf->Cell(0, 10, 'FECHA : ' . $fecha, 0, 1);
        $pdf->Cell(0, 10, 'MEDICO : ' . $medico, 0, 1);
        $pdf->Ln(5);

        // Generar el código de barras
        $generator = new BarcodeGeneratorPNG();
        $barcodeImage = WRITEPATH . 'cache/codigo_barras.png';
        file_put_contents($barcodeImage, $generator->getBarcode($folio, $generator::TYPE_CODE_128));

        // Insertar el código de barras en el PDF
        $pdf->Image($barcodeImage, 5, $pdf->GetY(), 60, 20); // Ajusta la posición y tamaño
        $pdf->Ln(20);
        // Mostrar el folio debajo del código de barras
        $pdf->SetFont('Arial', '', 12);
        $pdf->Cell(0, 5, '            Folio: ' . $folio, 0, 1, 1); // Texto centrado debajo del código de barras
        $pdf->Ln(5);

        // Generar el código QR
        /*   $qrCode = QrCode::create($idcita.' '.$nombre.''.$fecha);
        $writer = new PngWriter();
        $qrImagePath = WRITEPATH . 'cache/codigo_qr.png';
        $result = $writer->write($qrCode);
        $result->saveToFile($qrImagePath);
    
        // Insertar el código QR en el PDF
        $pdf->Image($qrImagePath, 10, $pdf->GetY(), 30, 30);
        $pdf->Ln(35); */

        // Diagnóstico
        $pdf->SetFont('Arial', 'B', 12);
        $pdf->Cell(0, 10, 'MOTIVO:', 0, 1);
        $pdf->SetFont('Arial', '', 12);
        $pdf->MultiCell(0, 10, $motivo);
        $pdf->Ln(5);

        // Limpiar las imágenes generadas
        if (file_exists($barcodeImage)) {
            unlink($barcodeImage);
        }
        /*   if (file_exists($qrImagePath)) {
            unlink($qrImagePath);
        } */

        $this->response->setHeader('Content-Type', 'application/pdf');
        $pdf->Output("ticket.pdf", "I");
    }



    public function insertar_prescripcion()
    {
        $id = $this->request->getPost('id_cita');

        if ($this->request->getMethod() == "post" && $this->validate($this->reglas_prescripcion)) {
            date_default_timezone_set('America/Caracas');
            $fecha = date('Y-m-d');
            $this->prescripcion->save([
                'id_paciente' => $this->request->getPost('id_paciente'),
                'id_personal' => $this->request->getPost('id_personal'),
                'id_medicamento' => $this->request->getPost('id_medicamento'),
                'dosis' => $this->request->getPost('dosis'),
                'indicaciones' => $this->request->getPost('indicaciones'),
                'fecha' => $fecha,
                'id_cita' => $this->request->getPost('id_cita')
            ]);

            return redirect()->to(base_url() . 'cita/atencion_cita/' . $id);
        } else {
            return $this->atencion_cita($id, $this->validator);
        }
    }

    public function eliminar_prescripcion($id, $id_cita)
    {
        $this->prescripcion->delete($id);

        return redirect()->to(base_url() . 'cita/atencion_cita/' . $id_cita);
    }

    public function actualizar_estado_cita()
    {
        $this->cita->update($this->request->getPost('id_cita'), [
            'estado_cita' => 0,
        ]);
        return redirect()->to(base_url() . 'cita');
    }
}
