<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use App\Models\EspecialidadModel;
use App\Models\DetallePermisosModel;
use App\Models\PermisosModel;

class Especialidad extends BaseController
{
    protected $especialidad, $session, $roles, $reglas, $reglasCambia, $permisos, $detalle_rol;

    public function __construct()
    {
        $this->especialidad = new EspecialidadModel();
        $this->detalle_rol = new DetallePermisosModel();
        $this->permisos = new PermisosModel();

        $this->session = session();

        helper(['form']);


        $this->reglas =
            [
                'nombre_especialidad' =>
                [
                    'rules' => 'required|is_unique[especialidad.nombre_especialidad]',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ]
            ];

        $this->reglasCambia =
            [
                'nombre_especialidad' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];
    }

    /* INICIO */
    public function index()
    {
        $especialidad = $this->especialidad->findAll();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


        $data = ['datos' => $especialidad,'validacion_sw' => 0, 'permisos' => $permisos];
        $view =
            view('header', $data) .
            view('especialidad/especialidad', $data) .
            view('footer');
        return $view;
    }
    public function insertar()
    {
        if ($this->request->getMethod() == "post" && $this->validate($this->reglas)) {

            $this->especialidad->save([
                'nombre_especialidad' => strtoupper($this->request->getPost('nombre_especialidad')),
                'estado_especialidad' => 1
            ]);

            return redirect()->to(base_url() . 'especialidad');
        } else {
            $especialidad = $this->especialidad->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $especialidad, 'validation' => $this->validator, 'validacion_sw' => 0, 'permisos' => $permisos];
            $view =
                view('header', $data) .
                view('especialidad/especialidad', $data) .
                view('footer');
            return $view;
        }
    }

    public function editar($id, $valid = null)
    {
        $especialidad = $this->especialidad->where('id_especialidad', $id)->first();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);


        if ($valid != null) {
            $data = ['datos' => $especialidad, 'validation' => $valid, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('especialidad/editar', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $especialidad, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('especialidad/editar', $data);
            echo view('footer');
        }
    }

    public function actualizar()
    {
        $id = $this->request->getPost('id_especialidad');
        $nombre_especialidad = $this->request->getPost('nombre_especialidad');
        $datos = $this->especialidad->where('id_especialidad !=', $id)->where('nombre_especialidad', $nombre_especialidad)->first();
        if (!$datos) {

            if ($this->request->getMethod() == "post" && $this->validate($this->reglasCambia)) {

                $this->especialidad->update($this->request->getPost('id_especialidad'), [
                    'nombre_especialidad' => strtoupper($this->request->getPost('nombre_especialidad')),
                    'estado_especialidad' => $this->request->getPost('estado_especialidad')

                ]);

                return redirect()->to(base_url() . 'especialidad');
            } else {

                return $this->editar($this->request->getPost('id_especialidad'), $this->validator);
            }
        } else {
  
            $especialidad = $this->especialidad->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $especialidad, 'validacion_sw' => 1, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('especialidad/especialidad', $data);
            echo view('footer');
        }
    }
}
