<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use App\Models\LaboratorioModel;
use App\Models\DetallePermisosModel;
use App\Models\PermisosModel;

class Laboratorio extends BaseController
{
    protected $laboratorio, $session, $roles, $reglas, $reglasCambia, $permisos, $detalle_rol;

    public function __construct()
    {
        $this->laboratorio = new LaboratorioModel();
        $this->detalle_rol = new DetallePermisosModel();
        $this->permisos = new PermisosModel();

        $this->session = session();

        helper(['form']);


        $this->reglas =
            [
                'nombre' =>
                [
                    'rules' => 'required|is_unique[laboratorio.nombre_laboratorio]',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'direccion' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'telefono' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];

        $this->reglasCambia =
            [
                'nombre' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'direccion' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'telefono' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];
    }

    /* INICIO */
    public function index()
    {
        $laboratorio = $this->laboratorio->findAll();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        $data = ['datos' => $laboratorio, 'validacion_sw' => 0, 'permisos' => $permisos];
        $view =
            view('header', $data) .
            view('laboratorio/laboratorio', $data) .
            view('footer');
        return $view;
    }
    public function insertar()
    {
        if ($this->request->getMethod() == "post" && $this->validate($this->reglas)) {

            $this->laboratorio->save([
                'nombre_laboratorio' => strtoupper($this->request->getPost('nombre')),
                'direccion' => $this->request->getPost('direccion'),
                'telefono' => $this->request->getPost('telefono'),
            ]);

            return redirect()->to(base_url() . 'laboratorio');
        } else {
            $laboratorio = $this->laboratorio->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $laboratorio, 'validation' => $this->validator, 'validacion_sw' => 0, 'permisos' => $permisos];
            $view =
                view('header', $data) .
                view('laboratorio/laboratorio', $data) .
                view('footer');
            return $view;
        }
    }

    public function editar($id, $valid = null)
    {
        $laboratorio = $this->laboratorio->where('id_laboratorio', $id)->first();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        if ($valid != null) {
            $data = ['datos' => $laboratorio, 'validation' => $valid, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('laboratorio/editar', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $laboratorio, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('laboratorio/editar', $data);
            echo view('footer');
        }
    }

    public function actualizar()
    {
        $id = $this->request->getPost('id_laboratorio');
        $nombre_laboratorio = $this->request->getPost('nombre');
        $datos = $this->laboratorio->where('id_laboratorio !=', $id)->where('nombre_laboratorio', $nombre_laboratorio)->first();
        if (!$datos) {

            if ($this->request->getMethod() == "post" && $this->validate($this->reglasCambia)) {

                $this->laboratorio->update($this->request->getPost('id_laboratorio'), [
                    'nombre_laboratorio' => strtoupper($this->request->getPost('nombre')),
                    'direccion' => $this->request->getPost('direccion'),
                    'telefono' => $this->request->getPost('telefono'),

                ]);

                return redirect()->to(base_url() . 'laboratorio');
            } else {

                return $this->editar($this->request->getPost('id_laboratorio'), $this->validator);
            }
        } else {

            $laboratorio = $this->laboratorio->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $laboratorio, 'validacion_sw' => 1, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('laboratorio/laboratorio', $data);
            echo view('footer');
        }
    }
}
