<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use App\Models\MedicamentoModel;
use App\Models\DetallePermisosModel;
use App\Models\PermisosModel;

class Medicamento extends BaseController
{
    protected $medicamento, $session, $roles, $reglas, $reglasCambia, $permisos, $detalle_rol;

    public function __construct()
    {
        $this->medicamento = new MedicamentoModel();
        $this->detalle_rol = new DetallePermisosModel();
        $this->permisos = new PermisosModel();

        $this->session = session();

        helper(['form']);


        $this->reglas =
            [
                'nombre_medicamento' =>
                [
                    'rules' => 'required|is_unique[medicamento.nombre_medicamento]',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                        'is_unique' => 'El campo {field} debe ser unico.'
                    ]
                ],
                'marca' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
                    ];

        $this->reglasCambia =
            [
                'nombre_medicamento' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'marca' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ],
                'estado_medicamento' =>
                [
                    'rules' => 'required',
                    'errors' => [
                        'required' => 'El campo {field} es obligatorio.',
                    ]
                ]
            ];
    }

    /* INICIO */
    public function index()
    {
        $medicamento = $this->medicamento->findAll();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        $data = ['datos' => $medicamento, 'validacion_sw' => 0, 'permisos' => $permisos];
        $view =
            view('header', $data) .
            view('medicamento/medicamento', $data) .
            view('footer');
        return $view;
    }
    public function insertar()
    {
        if ($this->request->getMethod() == "post" && $this->validate($this->reglas)) {

            $this->medicamento->save([
                'nombre_medicamento' => strtoupper($this->request->getPost('nombre_medicamento')),
                'marca' => $this->request->getPost('marca'),
                'estado_medicamento' => 1,
            ]);

            return redirect()->to(base_url() . 'medicamento');
        } else {
            $medicamento = $this->medicamento->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $medicamento, 'validation' => $this->validator, 'validacion_sw' => 0, 'permisos' => $permisos];
            $view =
                view('header', $data) .
                view('medicamento/medicamento', $data) .
                view('footer');
            return $view;
        }
    }

    public function editar($id, $valid = null)
    {
        $medicamento = $this->medicamento->where('id_medicamento', $id)->first();
        $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

        if ($valid != null) {
            $data = ['datos' => $medicamento, 'validation' => $valid, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('medicamento/editar', $data);
            echo view('footer');
        } else {
            $data = ['datos' => $medicamento, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('medicamento/editar', $data);
            echo view('footer');
        }
    }

    public function actualizar()
    {
        $id = $this->request->getPost('id_medicamento');
        $nombre_medicamento = $this->request->getPost('nombre_medicamento');
        $datos = $this->medicamento->where('id_medicamento !=', $id)->where('nombre_medicamento', $nombre_medicamento)->first();
        if (!$datos) {

            if ($this->request->getMethod() == "post" && $this->validate($this->reglasCambia)) {

                $this->medicamento->update($this->request->getPost('id_medicamento'), [
                    'nombre_medicamento' => strtoupper($this->request->getPost('nombre_medicamento')),
                    'marca' => $this->request->getPost('marca'),
                    'estado_medicamento' => $this->request->getPost('estado_medicamento'),

                ]);

                return redirect()->to(base_url() . 'medicamento');
            } else {

                return $this->editar($this->request->getPost('id_medicamento'), $this->validator);
            }
        } else {

            $medicamento = $this->medicamento->findAll();
            $permisos = $this->permisos->nombre_permisos($this->session->id_cargo);

            $data = ['datos' => $medicamento, 'validacion_sw' => 1, 'permisos' => $permisos];
            echo view('header', $data);
            echo view('medicamento/medicamento', $data);
            echo view('footer');
        }
    }
}
